from .__version__ import __version__
from .ui import ALMASimulator
from ._run import run

__doc__ = """
    ALMASim an elegant simulator for the ALMA Observatory for a more civilized age.

    ALMASim is a package for simulating the ALMA Observatory.
    It is a graphical user interface that allows users to simulate
    he ALMA Observatory and its observations.
    It is designed to be user-friendly and easy to use.
"""
